#!/usr/bin/perl -w
#
# DESC		: Utility to parse for Datetime in (at least) Canon m(2)ts Files
# VERSION	: 0.1
# CREATED	: 28.01.2009
# EDITED	: 28.01.2009
# AUTHOR	: Martin Strigl <mst@friendlynet.at>
# LICENCE	: GPL
#

use strict;
use Getopt::Long;
use POSIX;
use FileHandle;
use Time::Local;

$|=1;

my $input='';
my $packs=192;
my $split=0x47;
my $startread=1024;
my $maxsearch=100;
my $srch=chr(0x66).chr(0x4D).chr(0x44).chr(0x50).chr(0x4D);
my $dateformat='%Y/%m/%d';
my $timeformat='%H:%M:%S';

my $srchp='';
my $buffer='';
my $pos=-1;
my $filesize=0;
my $searchdone=0;
my $datum_start=40001231235959;
my $datum_ende=0;
my $vc=0;
my $startpos=0;
my $ts_start=0;
my $ts_ende=0;

GetOptions(
        'input=s'       =>      \$input,
        'packs:i'       =>      \$packs,
        'split:o'       =>      \$split,
	'reads:i'	=>	\$startread,
	'maxse:i'	=>	\$maxsearch,
	'chrss:s'	=>	\$srchp,
	'datef:s'	=>	\$dateformat,
	'timef:s'	=>	\$timeformat,
);
if (length($srchp)>0) {
	my @xy=split(/\s+/,$srchp);
	my $tmp='';
	for (@xy) {
		$tmp.=chr(eval($_));
	}
	$srch=$tmp;
}


sub OpenMax {
    my $openmax = POSIX::sysconf( &POSIX::_SC_OPEN_MAX );
    (!defined($openmax) || $openmax < 0) ? 2048 : $openmax;
}
 
sub ende {
	my $msg=shift;
	print $msg,"\n\n";
	foreach my $fhi (0 .. OpenMax) {
		# alle moeglicherweise offenen filehandle schliessen
	        POSIX::close($fhi);
	}
	exit;
}


unless ((-f $input) && (-s $input)) {
&ende(sprintf("USAGE: %s 

-i[nput]=MPEG2_TS_FILE			# required
-p[acks]=PACKETSIZE			# optional, integer, 			default %d
-s[plit]=SPLITBYTE_IN_HEX_NOTATION	# optional, hex (eg: 0x47), 		default 0x%X
-r[eads]=BYTES_TO_READ_AT_START		# optional, integer, 			default %d
-m[axse]=MAX_PACKETS_TO_SCAN_FOR_DATE	# optional, integer, 			default %d
-c[hrss]=CHR_SEQUENCE_TO_SCAN_FOR	# optional, hexlist (eg: 0x34 0x35),	default '0x%X 0x%X 0x%X 0x%X 0x%X'
-d[atef]=DATEFORMAT			# optional, string (see date command),	default '%s'
-t[imef]=TIMEFORMAT			# optional, string (see date command),	default '%s'

",$0,$packs,$split,$startread,$maxsearch,map(ord,split(//,$srch)),$dateformat,$timeformat));
}

$filesize=-s $input;
open(INF, $input) || &ende("Couldn't open file ".$input." because: ".$!);

# read the first $startread bytes. there we should find the splitbyte at least once, and then search for it
read(INF, $buffer, $startread);

$pos=index($buffer,chr($split));
$startpos=$pos;
if ($pos>=0) {
	# set fileposition to place of splitbyte
	seek(INF,$pos,0);
	# read in one packet of size $packs, check if the first byte matches splitbyte, and do it for maximal $maxsearch times
	while ( (read(INF,$buffer,$packs)==$packs) && ($searchdone<=$maxsearch) ) {
		if (index($buffer,chr($split))==0) {
			$vc++;
			# search for the $srch pattern, and when found extract the datetime value
			my $strt=index($buffer,$srch);
			if ($strt>=0) {
				$searchdone++;
				my @x=split(//,substr($buffer,$strt+8,8));
				my @dat=map(sprintf("%02X",ord($_)),@x);
				my $tmp=($dat[0]*100) + ($dat[1]);
				$tmp.=$dat[2];
				$tmp.=$dat[4];
				$tmp.=$dat[5];
				$tmp.=$dat[6];
				$tmp.=$dat[7];
				$datum_start=$tmp if ($tmp<$datum_start);
			}
		}
	}
	&ende("Didn't find the specified Search Pattern.") if ($searchdone==0);
	seek(INF,$vc*$packs*-1,2);
	seek(INF,$startpos,1);
	my $endstartpos=tell(INF);
	$vc=0;
	$searchdone=0;
	read(INF, $buffer, $startread);
	$pos=index($buffer,chr($split));
	seek(INF,$endstartpos+$pos,0);
	while ( (read(INF,$buffer,$packs)==192) && ($searchdone<=$maxsearch) ) {
		if (index($buffer,chr($split))==0) {
			$vc++;
			my $strt=index($buffer,$srch);
			if ($strt>=0) {
				$searchdone++;
				my @x=split(//,substr($buffer,$strt+8,8));
				my @dat=map(sprintf("%02X",ord($_)),@x);
				my $tmp=($dat[0]*100) + ($dat[1]);
				$tmp.=$dat[2];
				$tmp.=$dat[4];
				$tmp.=$dat[5];
				$tmp.=$dat[6];
				$tmp.=$dat[7];
				$datum_ende=$tmp if ($tmp>$datum_ende);
			}
		}
	}
	&ende("Didn't find the specified Search Pattern.") if ($searchdone==0);
} else {
	&ende("Couldn't find splitbyte in the first ".$startread." bytes.");
}
close(INF);



my @dates;
push(@dates,substr($datum_start,0,4));
push(@dates,substr($datum_start,4,2));
push(@dates,substr($datum_start,6,2));
push(@dates,substr($datum_start,8,2));
push(@dates,substr($datum_start,10,2));
push(@dates,substr($datum_start,12,2));
foreach my $i (0 .. 5) {
	$dates[$i]=~s/^0//;
}
my $monats=$dates[1];
$monats-=1;
$ts_start=timelocal($dates[5],$dates[4],$dates[3],$dates[2],$monats,$dates[0]);

my @datee;
push(@datee,substr($datum_ende,0,4));
push(@datee,substr($datum_ende,4,2));
push(@datee,substr($datum_ende,6,2));
push(@datee,substr($datum_ende,8,2));
push(@datee,substr($datum_ende,10,2));
push(@datee,substr($datum_ende,12,2));
foreach my $i (0 .. 5) {
	$datee[$i]=~s/^0//;
}
my $monate=$datee[1];
$monate-=1;
$ts_ende=timelocal($datee[5],$datee[4],$datee[3],$datee[2],$monate,$datee[0]);

my $aus_a=`date --date='$dates[0]-$dates[1]-$dates[2] $dates[3]:$dates[4]:$dates[5]' +'$dateformat'`;
my $aus_b=`date --date='$dates[0]-$dates[1]-$dates[2] $dates[3]:$dates[4]:$dates[5]' +'$timeformat'`;
my $aus_c=`date --date='$datee[0]-$datee[1]-$datee[2] $datee[3]:$datee[4]:$datee[5]' +'$dateformat'`;
my $aus_d=`date --date='$datee[0]-$datee[1]-$datee[2] $datee[3]:$datee[4]:$datee[5]' +'$timeformat'`;
$aus_a=~s/\n|\r//g;
$aus_b=~s/\n|\r//g;
$aus_c=~s/\n|\r//g;
$aus_d=~s/\n|\r//g;

print "DATE_START==",$aus_a,"\n";
print "TIME_START==",$aus_b,"\n";
print "DATE_CLOSE==",$aus_c,"\n";
print "TIME_CLOSE==",$aus_d,"\n";
print "VIDELENGTH==",($ts_ende-$ts_start),"\n";


